﻿using System;
using System.Web;
using System.Web.SessionState;
using Elmah;

/// <summary>
/// An <see cref="IRequestAuthorizationHandler"/> implementation that defers
/// authorization prior to execution of the request handler.
/// </summary>
/// <remarks>
/// For some scenarios, ELMAH may be calling <see cref="IRequestAuthorizationHandler.Authorize"/>
/// too early. If the authorization requires session state to be available,
/// for example, then this implementation can be used instead (provided that 
/// the handler implements <see cref="IRequiresSessionState"/> or 
/// <see cref="IReadOnlySessionState"/>) by overriding its 
/// <see cref="CompleteAuthorization"/> member.
/// </remarks>

public class ElmahAuthorizationHandler : HttpModuleBase, IRequestAuthorizationHandler
{
    protected override void OnInit(HttpApplication application)
    {
        application.PreRequestHandlerExecute += delegate 
        {
            var context = application.Context;
            if (IsFlaggedRequest(context))
                CompleteAuthorization(context);
        };

        base.OnInit(application);
    }

    protected override bool SupportDiscoverability
    {
        get { return true; }
    }

    public virtual bool Authorize(HttpContext context)
    {
        FlagRequest(context);
        return true;
    }

    protected virtual void CompleteAuthorization(HttpContext context)
    {
        if (context == null) throw new ArgumentNullException("context");
        
        // Example here forbids more than 100 requests
        // within a given session.
        //
        // if (context.Session == null)
        //     throw new Exception("Session unavailable.");
        // var count = (int)(context.Session["ElmahRequestCount"] ?? 0) + 1;
        // if (count > 100)
        //     throw new HttpException(403, "Unauthorized request.");
        // context.Session["ElmahRequestCount"] = count;
    }

    protected virtual void FlagRequest(HttpContext context)
    {
        if (context == null) throw new ArgumentNullException("context");
        context.Items[GetType()] = true;
    }

    protected virtual bool IsFlaggedRequest(HttpContext context)
    {
        if (context == null) throw new ArgumentNullException("context");
        return (bool)(context.Items[GetType()] ?? false);
    }
}
